import mock
import unittest
import os
import tempfile
from charmhelpers.contrib.cloudfoundry import contexts


class TestNatsContext(unittest.TestCase):

    @mock.patch('charmhelpers.core.hookenv.relation_ids')
    def test_nats_context_empty(self, mid):
        mid.return_value = None
        n = contexts.NatsContext()
        self.assertEqual(n(), {})

    @mock.patch('charmhelpers.core.hookenv.related_units')
    @mock.patch('charmhelpers.core.hookenv.relation_ids')
    @mock.patch('charmhelpers.core.hookenv.relation_get')
    def test_nats_context_populated(self, mrel, mid, mrelated):
        mid.return_value = ['nats']
        mrel.return_value = {'nats_port': 1234, 'nats_address': 'host',
                             'nats_user': 'user', 'nats_password': 'password'}
        mrelated.return_value = ['router/0']
        n = contexts.NatsContext()
        expected = {'nats': {'nats_port': 1234, 'nats_address': 'host',
                             'nats_user': 'user', 'nats_password': 'password'}}
        self.assertEqual(n(), expected)

    @mock.patch('charmhelpers.core.hookenv.related_units')
    @mock.patch('charmhelpers.core.hookenv.relation_ids')
    @mock.patch('charmhelpers.core.hookenv.relation_get')
    def test_nats_context_partial(self, mrel, mid, mrelated):
        mid.return_value = ['nats']
        mrel.return_value = {'nats_address': 'host'}
        mrelated.return_value = ['router/0']
        n = contexts.NatsContext()
        self.assertEqual(n(), {})


class TestRouterContext(unittest.TestCase):

    @mock.patch('charmhelpers.core.hookenv.relation_ids')
    def test_router_context_empty(self, mid):
        mid.return_value = None
        n = contexts.RouterContext()
        self.assertEqual(n(), {})

    @mock.patch('charmhelpers.core.hookenv.related_units')
    @mock.patch('charmhelpers.core.hookenv.relation_ids')
    @mock.patch('charmhelpers.core.hookenv.relation_get')
    def test_router_context_populated(self, mrel, mid, mrelated):
        mid.return_value = ['router']
        mrel.return_value = {'domain': 'example.com'}
        mrelated.return_value = ['router/0']
        n = contexts.RouterContext()
        expected = {'router': {'domain': 'example.com'}}
        self.assertEqual(n(), expected)


class TestConfigContext(unittest.TestCase):
    @mock.patch('charmhelpers.core.hookenv.config')
    def test_config_context(self, mconfig):
        contexts.ConfigContext()()
        self.assertTrue(mconfig.called)


class TestStoredContext(unittest.TestCase):

    def test_context_saving(self):
        _, file_name = tempfile.mkstemp()
        os.unlink(file_name)
        self.assertFalse(os.path.isfile(file_name))
        contexts.StoredContext(file_name, {'key': 'value'})
        self.assertTrue(os.path.isfile(file_name))

    def test_restoring(self):
        _, file_name = tempfile.mkstemp()
        os.unlink(file_name)
        contexts.StoredContext(file_name, {'key': 'initial_value'})
        self.assertTrue(os.path.isfile(file_name))
        context = contexts.StoredContext(file_name, {'key': 'random_value'})()
        self.assertIn('key', context)
        self.assertEqual(context['key'], 'initial_value')

    def test_stored_context_raise(self):
        _, file_name = tempfile.mkstemp()
        with self.assertRaises(OSError) as cm:
            contexts.StoredContext(file_name, {'key': 'initial_value'})
        os.unlink(file_name)

    def test_static_context(self):
        a = contexts.StaticContext('a')
        self.assertEqual(a.data, 'a')
        self.assertEqual(a(), 'a')

if __name__ == '__main__':
    unittest.main()
