import pkg_resources
import mock
import unittest
from charmhelpers.contrib.cloudfoundry import services

CF = 'charmhelpers.contrib.cloudfoundry'
SERVICES = CF + '.services'
TEMPLATES_DIR = pkg_resources.resource_filename(__name__, 'templates')

# Test methods for context collection
def noop():
    return {}


def found():
    return {'foo': 'bar'}


def static_content(data):
    def context():
        return data
    return context


# Sample render context used to test rendering paths
default_context = {
    'nats': {
        'nats_port': '1234',
        'nats_host': 'example.com',
    },
    'router': {
        'domain': 'api.foo.com'
    }
}


# Method returning a mutable copy of a default config
# for services.register()
def default_service_config():
    return [{
        'service': 'cf-cloudcontroller',
        'templates': [
            {'target': 'cc.yml',
             'source': 'fake_cc.yml',
             'file_properties': {},
             'contexts': [static_content(default_context)]
             }]
    }]


class TestService(unittest.TestCase):
    def setUp(self):
        services.reset()

    @mock.patch(SERVICES + '.reconfigure_service')
    def test_reconfigure_services(self, mreconfig):
        services.register(default_service_config(), TEMPLATES_DIR)
        services.reconfigure_services()
        # Verify that we called reconfigure_service(name, restart=True)
        self.assertEqual(mreconfig.call_args[0][0], 'cf-cloudcontroller')
        self.assertTrue(mreconfig.call_args[1]['restart'])

    @mock.patch(SERVICES + '.install')
    @mock.patch('charmhelpers.core.host.service_restart')
    def test_reconfigure_service_restart(self, mrestart, minstall):
        services.register(default_service_config(), TEMPLATES_DIR)
        services.reconfigure_service('cf-cloudcontroller', restart=True)
        # Verify that we called service_restart(name)
        self.assertEqual(mrestart.call_args[0][0], 'cf-cloudcontroller')
        self.assertTrue(mrestart.called)

    @mock.patch('os.unlink')
    @mock.patch(SERVICES + '.render_template')
    @mock.patch(SERVICES + '.install')
    def test_reconfigure_service_no_source(self, minstall, mrender, munlink):
        config = default_service_config()
        # erase the contexts
        config[0]['templates'][0]['contexts'] = {}
        services.register(config, TEMPLATES_DIR)
        services.reconfigure_service('cf-cloudcontroller', restart=True)
        # verify that we not called render template
        self.assertFalse(mrender.called)
        self.assertFalse(minstall.called)
        self.assertFalse(munlink.called)

    @mock.patch('os.unlink')
    @mock.patch(SERVICES + '.render_template')
    @mock.patch(SERVICES + '.install')
    def test_reconfigure_service_no_context(self, minstall, mrender, munlink):
        config = default_service_config()
        # delete the source so the lookup pattern is used
        del config[0]['templates'][0]['source']
        services.register(config, TEMPLATES_DIR)
        services.reconfigure_service('cf-cloudcontroller', restart=False)
        # verify that we called render template with the expected name
        self.assertEqual(mrender.call_args[0][0], 'cc.yml.j2')
        self.assertTrue(minstall.called)
        self.assertTrue(munlink.called)
        self.assertRaises(KeyError, services.reconfigure_service, 'unknownservice', restart=False)

               
    def test_render_template(self):
        services.configure_templates(TEMPLATES_DIR)
        output = services.render_template('fake_cc.yml', default_context)
        contents = open(output).read()
        self.assertRegexpMatches(contents, 'port: 1234')
        self.assertRegexpMatches(contents, 'host: example.com')
        self.assertRegexpMatches(contents, 'domain: api.foo.com')

    def test_collect_contexts_fail(self):
        cc = services.collect_contexts
        self.assertEqual(cc([]), {})
        self.assertEqual(cc([noop]), {})
        self.assertEqual(cc([noop, noop]), {})
        self.assertEqual(cc([noop, found]), {})

    def test_collect_contexts_found(self):
        cc = services.collect_contexts
        expected = {'foo': 'bar'}
        self.assertEqual(cc([found]), expected)
        self.assertEqual(cc([found, found]), expected)

    @mock.patch('charmhelpers.core.host.service_stop')
    @mock.patch('charmhelpers.core.host.service_running')
    def test_stop_services(self, mrunning, mstop):
        services.register(default_service_config(), TEMPLATES_DIR)
        services.stop_services()
        self.assertTrue(mrunning.call_args[0], 'cf-cloudcontroller')
        self.assertTrue(mstop.call_args[0], 'cf-cloudcontroller')
    
    def test_get_service(self):
        services.register(default_service_config(), TEMPLATES_DIR)
        service = services.get_service('cf-cloudcontroller')
        self.assertEqual(service['service'], 'cf-cloudcontroller')

    def test_get_service_none(self):
        services.register(default_service_config(), TEMPLATES_DIR)
        self.assertIsNone(services.get_service('non_existing_one'))

if __name__ == '__main__':
    unittest.main()
